@@ -77,7 +77,7 @@ struct as_list {
 	struct as_filter *tail;
 };
 
-/* ip as-path access-list 10 permit AS1. */
+/* as-path access-list 10 permit AS1. */
 
 static struct as_list_master as_list_master = {{NULL, NULL},
 					       {NULL, NULL},
@@ -95,8 +95,7 @@ static void as_filter_free(struct as_filter *asfilter)
 {
 	if (asfilter->reg)
 		bgp_regex_free(asfilter->reg);
-	if (asfilter->reg_str)
-		XFREE(MTYPE_AS_FILTER_STR, asfilter->reg_str);
+	XFREE(MTYPE_AS_FILTER_STR, asfilter->reg_str);
 	XFREE(MTYPE_AS_FILTER, asfilter);
 }
 
@@ -169,10 +168,7 @@ static struct as_list *as_list_new(void)
 
 static void as_list_free(struct as_list *aslist)
 {
-	if (aslist->name) {
-		XFREE(MTYPE_AS_STR, aslist->name);
-		aslist->name = NULL;
-	}
+	XFREE(MTYPE_AS_STR, aslist->name);
 	XFREE(MTYPE_AS_LIST, aslist);
 }
 
@@ -194,7 +190,7 @@ static struct as_list *as_list_insert(const char *name)
 	/* If name is made by all digit character.  We treat it as
 	   number. */
 	for (number = 0, i = 0; i < strlen(name); i++) {
-		if (isdigit((int)name[i]))
+		if (isdigit((unsigned char)name[i]))
 			number = (number * 10) + (name[i] - '0');
 		else
 			break;
@@ -306,12 +302,9 @@ static void as_list_delete(struct as_list *aslist)
 	as_list_free(aslist);
 }
 
-static int as_list_empty(struct as_list *aslist)
+static bool as_list_empty(struct as_list *aslist)
 {
-	if (aslist->head == NULL && aslist->tail == NULL)
-		return 1;
-	else
-		return 0;
+	return aslist->head == NULL && aslist->tail == NULL;
 }
 
 static void as_list_filter_delete(struct as_list *aslist,
@@ -338,15 +331,12 @@ static void as_list_filter_delete(struct as_list *aslist,
 	/* Run hook function. */
 	if (as_list_master.delete_hook)
 		(*as_list_master.delete_hook)(name);
-	if (name)
-		XFREE(MTYPE_AS_STR, name);
+	XFREE(MTYPE_AS_STR, name);
 }
 
-static int as_filter_match(struct as_filter *asfilter, struct aspath *aspath)
+static bool as_filter_match(struct as_filter *asfilter, struct aspath *aspath)
 {
-	if (bgp_regexec(asfilter->reg, aspath) != REG_NOMATCH)
-		return 1;
-	return 0;
+	return bgp_regexec(asfilter->reg, aspath) != REG_NOMATCH;
 }
 
 /* Apply AS path filter to AS. */
@@ -379,28 +369,36 @@ void as_list_delete_hook(void (*func)(const char *))
 	as_list_master.delete_hook = func;
 }
 
-static int as_list_dup_check(struct as_list *aslist, struct as_filter *new)
+static bool as_list_dup_check(struct as_list *aslist, struct as_filter *new)
 {
 	struct as_filter *asfilter;
 
 	for (asfilter = aslist->head; asfilter; asfilter = asfilter->next) {
 		if (asfilter->type == new->type
 		    && strcmp(asfilter->reg_str, new->reg_str) == 0)
-			return 1;
+			return true;
 	}
-	return 0;
+	return false;
 }
 
-DEFUN (ip_as_path,
-       ip_as_path_cmd,
-       "ip as-path access-list WORD <deny|permit> LINE...",
-       IP_STR
-       "BGP autonomous system path filter\n"
-       "Specify an access list name\n"
-       "Regular expression access list name\n"
-       "Specify packets to reject\n"
-       "Specify packets to forward\n"
-       "A regular-expression to match the BGP AS paths\n")
+bool config_bgp_aspath_validate(const char *regstr)
+{
+	char valid_chars[] = "1234567890_^|[,{}() ]$*+.?-\\";
+
+	if (strspn(regstr, valid_chars) == strlen(regstr))
+		return true;
+	return false;
+}
+
+DEFUN(as_path, bgp_as_path_cmd,
+      "bgp as-path access-list WORD <deny|permit> LINE...",
+      BGP_STR
+      "BGP autonomous system path filter\n"
+      "Specify an access list name\n"
+      "Regular expression access list name\n"
+      "Specify packets to reject\n"
+      "Specify packets to forward\n"
+      "A regular-expression (1234567890_^|[,{}() ]$*+.?-\\) to match the BGP AS paths\n")
 {
 	int idx = 0;
 	enum as_filter_type type;
@@ -428,6 +426,12 @@ DEFUN (ip_as_path,
 		return CMD_WARNING_CONFIG_FAILED;
 	}
 
+	if (!config_bgp_aspath_validate(regstr)) {
+		vty_out(vty, "Invalid character in as-path access-list %s\n",
+			regstr);
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+
 	asfilter = as_filter_make(regex, regstr, type);
 
 	XFREE(MTYPE_TMP, regstr);
@@ -444,17 +448,16 @@ DEFUN (ip_as_path,
 	return CMD_SUCCESS;
 }
 
-DEFUN (no_ip_as_path,
-       no_ip_as_path_cmd,
-       "no ip as-path access-list WORD <deny|permit> LINE...",
-       NO_STR
-       IP_STR
-       "BGP autonomous system path filter\n"
-       "Specify an access list name\n"
-       "Regular expression access list name\n"
-       "Specify packets to reject\n"
-       "Specify packets to forward\n"
-       "A regular-expression to match the BGP AS paths\n")
+DEFUN(no_as_path, no_bgp_as_path_cmd,
+      "no bgp as-path access-list WORD <deny|permit> LINE...",
+      NO_STR
+      BGP_STR
+      "BGP autonomous system path filter\n"
+      "Specify an access list name\n"
+      "Regular expression access list name\n"
+      "Specify packets to reject\n"
+      "Specify packets to forward\n"
+      "A regular-expression (1234567890_^|[,{}() ]$*+.?-\\) to match the BGP AS paths\n")
 {
 	int idx = 0;
 	enum as_filter_type type;
@@ -469,7 +472,7 @@ DEFUN (no_ip_as_path,
 	/* Lookup AS list from AS path list. */
 	aslist = as_list_lookup(aslistname);
 	if (aslist == NULL) {
-		vty_out(vty, "ip as-path access-list %s doesn't exist\n",
+		vty_out(vty, "bgp as-path access-list %s doesn't exist\n",
 			aslistname);
 		return CMD_WARNING_CONFIG_FAILED;
 	}
@@ -488,6 +491,12 @@ DEFUN (no_ip_as_path,
 	argv_find(argv, argc, "LINE", &idx);
 	regstr = argv_concat(argv, argc, idx);
 
+	if (!config_bgp_aspath_validate(regstr)) {
+		vty_out(vty, "Invalid character in as-path access-list %s\n",
+			regstr);
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+
 	regex = bgp_regcomp(regstr);
 	if (!regex) {
 		vty_out(vty, "can't compile regexp %s\n", regstr);
@@ -498,24 +507,26 @@ DEFUN (no_ip_as_path,
 	/* Lookup asfilter. */
 	asfilter = as_filter_lookup(aslist, regstr, type);
 
-	XFREE(MTYPE_TMP, regstr);
 	bgp_regex_free(regex);
 
 	if (asfilter == NULL) {
-		vty_out(vty, "\n");
+		vty_out(vty, "Regex entered %s does not exist\n", regstr);
+		XFREE(MTYPE_TMP, regstr);
 		return CMD_WARNING_CONFIG_FAILED;
 	}
 
+	XFREE(MTYPE_TMP, regstr);
+
 	as_list_filter_delete(aslist, asfilter);
 
 	return CMD_SUCCESS;
 }
 
-DEFUN (no_ip_as_path_all,
-       no_ip_as_path_all_cmd,
-       "no ip as-path access-list WORD",
+DEFUN (no_as_path_all,
+       no_bgp_as_path_all_cmd,
+       "no bgp as-path access-list WORD",
        NO_STR
-       IP_STR
+       BGP_STR
        "BGP autonomous system path filter\n"
        "Specify an access list name\n"
        "Regular expression access list name\n")
@@ -525,7 +536,7 @@ DEFUN (no_ip_as_path_all,
 
 	aslist = as_list_lookup(argv[idx_word]->arg);
 	if (aslist == NULL) {
-		vty_out(vty, "ip as-path access-list %s doesn't exist\n",
+		vty_out(vty, "bgp as-path access-list %s doesn't exist\n",
 			argv[idx_word]->arg);
 		return CMD_WARNING_CONFIG_FAILED;
 	}
@@ -579,11 +590,11 @@ static void as_list_show_all(struct vty *vty)
 	}
 }
 
-DEFUN (show_ip_as_path_access_list,
-       show_ip_as_path_access_list_cmd,
-       "show ip as-path-access-list WORD",
+DEFUN (show_as_path_access_list,
+       show_bgp_as_path_access_list_cmd,
+       "show bgp as-path-access-list WORD",
        SHOW_STR
-       IP_STR
+       BGP_STR
        "List AS path access lists\n"
        "AS path access list name\n")
 {
@@ -597,17 +608,32 @@ DEFUN (show_ip_as_path_access_list,
 	return CMD_SUCCESS;
 }
 
-DEFUN (show_ip_as_path_access_list_all,
-       show_ip_as_path_access_list_all_cmd,
-       "show ip as-path-access-list",
+ALIAS (show_as_path_access_list,
+       show_ip_as_path_access_list_cmd,
+       "show ip as-path-access-list WORD",
        SHOW_STR
        IP_STR
+       "List AS path access lists\n"
+       "AS path access list name\n")
+
+DEFUN (show_as_path_access_list_all,
+       show_bgp_as_path_access_list_all_cmd,
+       "show bgp as-path-access-list",
+       SHOW_STR
+       BGP_STR
        "List AS path access lists\n")
 {
 	as_list_show_all(vty);
 	return CMD_SUCCESS;
 }
 
+ALIAS (show_as_path_access_list_all,
+       show_ip_as_path_access_list_all_cmd,
+       "show ip as-path-access-list",
+       SHOW_STR
+       IP_STR
+       "List AS path access lists\n")
+
 static int config_write_as_list(struct vty *vty)
 {
 	struct as_list *aslist;
@@ -617,7 +643,7 @@ static int config_write_as_list(struct vty *vty)
 	for (aslist = as_list_master.num.head; aslist; aslist = aslist->next)
 		for (asfilter = aslist->head; asfilter;
 		     asfilter = asfilter->next) {
-			vty_out(vty, "ip as-path access-list %s %s %s\n",
+			vty_out(vty, "bgp as-path access-list %s %s %s\n",
 				aslist->name, filter_type_str(asfilter->type),
 				asfilter->reg_str);
 			write++;
@@ -626,7 +652,7 @@ static int config_write_as_list(struct vty *vty)
 	for (aslist = as_list_master.str.head; aslist; aslist = aslist->next)
 		for (asfilter = aslist->head; asfilter;
 		     asfilter = asfilter->next) {
-			vty_out(vty, "ip as-path access-list %s %s %s\n",
+			vty_out(vty, "bgp as-path access-list %s %s %s\n",
 				aslist->name, filter_type_str(asfilter->type),
 				asfilter->reg_str);
 			write++;
@@ -634,18 +660,26 @@ static int config_write_as_list(struct vty *vty)
 	return write;
 }
 
-static struct cmd_node as_list_node = {AS_LIST_NODE, "", 1};
+static int config_write_as_list(struct vty *vty);
+static struct cmd_node as_list_node = {
+	.name = "as list",
+	.node = AS_LIST_NODE,
+	.prompt = "",
+	.config_write = config_write_as_list,
+};
 
 /* Register functions. */
 void bgp_filter_init(void)
 {
-	install_node(&as_list_node, config_write_as_list);
+	install_node(&as_list_node);
 
-	install_element(CONFIG_NODE, &ip_as_path_cmd);
-	install_element(CONFIG_NODE, &no_ip_as_path_cmd);
-	install_element(CONFIG_NODE, &no_ip_as_path_all_cmd);
+	install_element(CONFIG_NODE, &bgp_as_path_cmd);
+	install_element(CONFIG_NODE, &no_bgp_as_path_cmd);
+	install_element(CONFIG_NODE, &no_bgp_as_path_all_cmd);
 
+	install_element(VIEW_NODE, &show_bgp_as_path_access_list_cmd);
 	install_element(VIEW_NODE, &show_ip_as_path_access_list_cmd);
+	install_element(VIEW_NODE, &show_bgp_as_path_access_list_all_cmd);
 	install_element(VIEW_NODE, &show_ip_as_path_access_list_all_cmd);
 }
 
