@@ -22,16 +22,41 @@
 #ifndef _ZEBRA_ISIS_ZEBRA_H
 #define _ZEBRA_ISIS_ZEBRA_H
 
+#include "isisd.h"
+
 extern struct zclient *zclient;
 
-void isis_zebra_init(struct thread_master *);
+struct label_chunk {
+	uint32_t start;
+	uint32_t end;
+	uint64_t used_mask;
+};
+#define CHUNK_SIZE 64
+
+void isis_zebra_init(struct thread_master *master, int instance);
 void isis_zebra_stop(void);
 
-void isis_zebra_route_update(struct prefix *prefix,
-			     struct prefix_ipv6 *src_p,
-			     struct isis_route_info *route_info);
+struct isis_route_info;
+struct sr_prefix;
+struct sr_adjacency;
+
+void isis_zebra_route_add_route(struct isis *isis,
+				struct prefix *prefix,
+				struct prefix_ipv6 *src_p,
+				struct isis_route_info *route_info);
+void isis_zebra_route_del_route(struct isis *isis,
+				struct prefix *prefix,
+				struct prefix_ipv6 *src_p,
+				struct isis_route_info *route_info);
+void isis_zebra_send_prefix_sid(int cmd, const struct sr_prefix *srp);
+void isis_zebra_send_adjacency_sid(int cmd, const struct sr_adjacency *sra);
 int isis_distribute_list_update(int routetype);
 void isis_zebra_redistribute_set(afi_t afi, int type);
 void isis_zebra_redistribute_unset(afi_t afi, int type);
+bool isis_zebra_label_manager_ready(void);
+int isis_zebra_label_manager_connect(void);
+int isis_zebra_request_label_range(uint32_t base, uint32_t chunk_size);
+int isis_zebra_release_label_range(uint32_t start, uint32_t end);
+void isis_zebra_vrf_register(struct isis *isis);
 
 #endif /* _ZEBRA_ISIS_ZEBRA_H */
