@@ -837,7 +837,7 @@ static uint8_t *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
 				  int exact, size_t *var_len,
 				  WriteMethod **write_method)
 {
-	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
+	struct vrf *vrf;
 	struct ospf6_lsa *lsa = NULL;
 	ifindex_t ifindex;
 	uint32_t area_id, id, instid, adv_router;
@@ -861,6 +861,7 @@ static uint8_t *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
 	if (ospf6 == NULL)
 		return NULL;
 
+	vrf = vrf_lookup_by_id(ospf6->vrf_id);
 	/* Get variable length. */
 	offset = name + v->namelen;
 	offsetlen = *length - v->namelen;
@@ -926,7 +927,8 @@ static uint8_t *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
 				return NULL;
 			lsa = ospf6_lsdb_lookup(type, id, adv_router, oa->lsdb);
 		} else if (v->magic & OSPFv3WWLINKTABLE) {
-			oi = ospf6_interface_lookup_by_ifindex(ifindex);
+			oi = ospf6_interface_lookup_by_ifindex(ifindex,
+							       ospf6->vrf_id);
 			if (!oi || oi->instance_id != instid)
 				return NULL;
 			lsa = ospf6_lsdb_lookup(type, id, adv_router, oi->lsdb);
@@ -963,7 +965,7 @@ static uint8_t *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
 				if (!iif->ifindex)
 					continue;
 				oi = ospf6_interface_lookup_by_ifindex(
-					iif->ifindex);
+					iif->ifindex, iif->vrf_id);
 				if (!oi)
 					continue;
 				if (iif->ifindex < ifindex)
@@ -1017,18 +1019,14 @@ static uint8_t *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
 	switch (v->magic & OSPFv3WWCOLUMN) {
 	case OSPFv3WWLSDBSEQUENCE:
 		return SNMP_INTEGER(ntohl(lsa->header->seqnum));
-		break;
 	case OSPFv3WWLSDBAGE:
 		ospf6_lsa_age_current(lsa);
 		return SNMP_INTEGER(ntohs(lsa->header->age));
-		break;
 	case OSPFv3WWLSDBCHECKSUM:
 		return SNMP_INTEGER(ntohs(lsa->header->checksum));
-		break;
 	case OSPFv3WWLSDBADVERTISEMENT:
 		*var_len = ntohs(lsa->header->length);
 		return (uint8_t *)lsa->header;
-		break;
 	case OSPFv3WWLSDBTYPEKNOWN:
 		return SNMP_INTEGER(OSPF6_LSA_IS_KNOWN(lsa->header->type)
 					    ? SNMP_TRUE
@@ -1042,7 +1040,7 @@ static uint8_t *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 			      int exact, size_t *var_len,
 			      WriteMethod **write_method)
 {
-	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
+	struct vrf *vrf;
 	ifindex_t ifindex = 0;
 	unsigned int instid = 0;
 	struct ospf6_interface *oi = NULL;
@@ -1062,6 +1060,7 @@ static uint8_t *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 	if (ospf6 == NULL)
 		return NULL;
 
+	vrf = vrf_lookup_by_id(ospf6->vrf_id);
 	/* Get variable length. */
 	offset = name + v->namelen;
 	offsetlen = *length - v->namelen;
@@ -1084,7 +1083,7 @@ static uint8_t *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 	// offsetlen -= len;
 
 	if (exact) {
-		oi = ospf6_interface_lookup_by_ifindex(ifindex);
+		oi = ospf6_interface_lookup_by_ifindex(ifindex, ospf6->vrf_id);
 		if (!oi || oi->instance_id != instid)
 			return NULL;
 	} else {
@@ -1099,7 +1098,8 @@ static uint8_t *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 		for (ALL_LIST_ELEMENTS_RO(ifslist, i, iif)) {
 			if (!iif->ifindex)
 				continue;
-			oi = ospf6_interface_lookup_by_ifindex(iif->ifindex);
+			oi = ospf6_interface_lookup_by_ifindex(iif->ifindex,
+							       iif->vrf_id);
 			if (!oi)
 				continue;
 			if (iif->ifindex > ifindex
@@ -1130,9 +1130,9 @@ static uint8_t *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 			return SNMP_INTEGER(ntohl(oi->area->area_id));
 		break;
 	case OSPFv3IFTYPE:
-		if (if_is_broadcast(oi->interface))
+		if (oi->type == OSPF_IFTYPE_BROADCAST)
 			return SNMP_INTEGER(1);
-		else if (if_is_pointopoint(oi->interface))
+		else if (oi->type == OSPF_IFTYPE_POINTOPOINT)
 			return SNMP_INTEGER(3);
 		else
 			break; /* Unknown, don't put anything */
@@ -1195,7 +1195,7 @@ static uint8_t *ospfv3NbrEntry(struct variable *v, oid *name, size_t *length,
 			       int exact, size_t *var_len,
 			       WriteMethod **write_method)
 {
-	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
+	struct vrf *vrf;
 	ifindex_t ifindex = 0;
 	unsigned int instid, rtrid;
 	struct ospf6_interface *oi = NULL;
@@ -1216,6 +1216,7 @@ static uint8_t *ospfv3NbrEntry(struct variable *v, oid *name, size_t *length,
 	if (ospf6 == NULL)
 		return NULL;
 
+	vrf = vrf_lookup_by_id(ospf6->vrf_id);
 	/* Get variable length. */
 	offset = name + v->namelen;
 	offsetlen = *length - v->namelen;
@@ -1245,7 +1246,7 @@ static uint8_t *ospfv3NbrEntry(struct variable *v, oid *name, size_t *length,
 	// offsetlen -= len;
 
 	if (exact) {
-		oi = ospf6_interface_lookup_by_ifindex(ifindex);
+		oi = ospf6_interface_lookup_by_ifindex(ifindex, ospf6->vrf_id);
 		if (!oi || oi->instance_id != instid)
 			return NULL;
 		on = ospf6_neighbor_lookup(rtrid, oi);
@@ -1261,7 +1262,8 @@ static uint8_t *ospfv3NbrEntry(struct variable *v, oid *name, size_t *length,
 		for (ALL_LIST_ELEMENTS_RO(ifslist, i, iif)) {
 			if (!iif->ifindex)
 				continue;
-			oi = ospf6_interface_lookup_by_ifindex(iif->ifindex);
+			oi = ospf6_interface_lookup_by_ifindex(iif->ifindex,
+							       iif->vrf_id);
 			if (!oi)
 				continue;
 			for (ALL_LIST_ELEMENTS_RO(oi->neighbor_list, j, on)) {
@@ -1358,13 +1360,10 @@ static int ospf6TrapNbrStateChange(struct ospf6_neighbor *on, int next_state,
 	index[1] = on->ospf6_if->instance_id;
 	index[2] = ntohl(on->router_id);
 
-	smux_trap(ospfv3_variables,
-		  sizeof ospfv3_variables / sizeof(struct variable),
-		  ospfv3_trap_oid, sizeof ospfv3_trap_oid / sizeof(oid),
-		  ospfv3_oid, sizeof ospfv3_oid / sizeof(oid), index, 3,
-		  ospf6NbrTrapList,
-		  sizeof ospf6NbrTrapList / sizeof(struct trap_object),
-		  NBRSTATECHANGE);
+	smux_trap(ospfv3_variables, array_size(ospfv3_variables),
+		  ospfv3_trap_oid, array_size(ospfv3_trap_oid), ospfv3_oid,
+		  sizeof(ospfv3_oid) / sizeof(oid), index, 3, ospf6NbrTrapList,
+		  array_size(ospf6NbrTrapList), NBRSTATECHANGE);
 	return 0;
 }
 
@@ -1383,13 +1382,10 @@ static int ospf6TrapIfStateChange(struct ospf6_interface *oi, int next_state,
 	index[0] = oi->interface->ifindex;
 	index[1] = oi->instance_id;
 
-	smux_trap(ospfv3_variables,
-		  sizeof ospfv3_variables / sizeof(struct variable),
-		  ospfv3_trap_oid, sizeof ospfv3_trap_oid / sizeof(oid),
-		  ospfv3_oid, sizeof ospfv3_oid / sizeof(oid), index, 2,
-		  ospf6IfTrapList,
-		  sizeof ospf6IfTrapList / sizeof(struct trap_object),
-		  IFSTATECHANGE);
+	smux_trap(ospfv3_variables, array_size(ospfv3_variables),
+		  ospfv3_trap_oid, array_size(ospfv3_trap_oid), ospfv3_oid,
+		  sizeof(ospfv3_oid) / sizeof(oid), index, 2, ospf6IfTrapList,
+		  array_size(ospf6IfTrapList), IFSTATECHANGE);
 	return 0;
 }
 
