@@ -23,6 +23,7 @@
 
 DECLARE_MGROUP(STATIC)
 
-DECLARE_MTYPE(STATIC_ROUTE);
+DECLARE_MTYPE(STATIC_NEXTHOP);
+DECLARE_MTYPE(STATIC_PATH);
 
 #endif
