@@ -32,7 +32,7 @@ from __future__ import print_function, unicode_literals
 import argparse
 import copy
 import logging
-import os
+import os, os.path
 import random
 import re
 import string
@@ -59,9 +59,110 @@ else:
 log = logging.getLogger(__name__)
 
 
-class VtyshMarkException(Exception):
+class VtyshException(Exception):
     pass
 
+class Vtysh(object):
+    def __init__(self, bindir=None, confdir=None, sockdir=None, pathspace=None):
+        self.bindir = bindir
+        self.confdir = confdir
+        self.pathspace = pathspace
+        self.common_args = [os.path.join(bindir or '', 'vtysh')]
+        if confdir:
+            self.common_args.extend(['--config_dir', confdir])
+        if sockdir:
+            self.common_args.extend(['--vty_socket', sockdir])
+        if pathspace:
+            self.common_args.extend(['-N', pathspace])
+
+    def _call(self, args, stdin=None, stdout=None, stderr=None):
+        kwargs = {}
+        if stdin is not None:
+            kwargs['stdin'] = stdin
+        if stdout is not None:
+            kwargs['stdout'] = stdout
+        if stderr is not None:
+            kwargs['stderr'] = stderr
+        return subprocess.Popen(self.common_args + args, **kwargs)
+
+    def _call_cmd(self, command, stdin=None, stdout=None, stderr=None):
+        if isinstance(command, list):
+            args = [item for sub in command for item in ['-c', sub]]
+        else:
+            args = ['-c', command]
+        return self._call(args, stdin, stdout, stderr)
+
+    def __call__(self, command):
+        """
+        Call a CLI command (e.g. "show running-config")
+
+        Output text is automatically redirected, decoded and returned.
+        Multiple commands may be passed as list.
+        """
+        proc = self._call_cmd(command, stdout=subprocess.PIPE)
+        stdout, stderr = proc.communicate()
+        if proc.wait() != 0:
+            raise VtyshException('vtysh returned status %d for command "%s"'
+                    % (proc.returncode, command))
+        return stdout.decode('UTF-8')
+
+    def is_config_available(self):
+        """
+        Return False if no frr daemon is running or some other vtysh session is
+        in 'configuration terminal' mode which will prevent us from making any
+        configuration changes.
+        """
+
+        output = self('configure')
+
+        if 'VTY configuration is locked by other VTY' in output:
+            log.error("vtysh 'configure' returned\n%s\n" % (output))
+            return False
+
+        return True
+
+    def exec_file(self, filename):
+        child = self._call(['-f', filename])
+        if child.wait() != 0:
+            raise VtyshException('vtysh (exec file) exited with status %d'
+                    % (child.returncode))
+
+    def mark_file(self, filename, stdin=None):
+        child = self._call(['-m', '-f', filename],
+                stdout=subprocess.PIPE, stdin=subprocess.PIPE, stderr=subprocess.PIPE)
+        try:
+            stdout, stderr = child.communicate()
+        except subprocess.TimeoutExpired:
+            child.kill()
+            stdout, stderr = child.communicate()
+            raise VtyshException('vtysh call timed out!')
+
+        if child.wait() != 0:
+            raise VtyshException('vtysh (mark file) exited with status %d:\n%s'
+                    % (child.returncode, stderr))
+
+        return stdout.decode('UTF-8')
+
+    def mark_show_run(self, daemon = None):
+        cmd = 'show running-config'
+        if daemon:
+            cmd += ' %s' % daemon
+        cmd += ' no-header'
+        show_run = self._call_cmd(cmd, stdout=subprocess.PIPE)
+        mark = self._call(['-m', '-f', '-'], stdin=show_run.stdout, stdout=subprocess.PIPE)
+
+        show_run.wait()
+        stdout, stderr = mark.communicate()
+        mark.wait()
+
+        if show_run.returncode != 0:
+            raise VtyshException('vtysh (show running-config) exited with status %d:'
+                    % (show_run.returncode))
+        if mark.returncode != 0:
+            raise VtyshException('vtysh (mark running-config) exited with status %d'
+                    % (mark.returncode))
+
+        return stdout.decode('UTF-8')
 
 class Context(object):
 
@@ -110,9 +211,10 @@ class Config(object):
     ('router ospf' for example) are our dictionary key.
     """
 
-    def __init__(self):
+    def __init__(self, vtysh):
         self.lines = []
         self.contexts = OrderedDict()
+        self.vtysh = vtysh
 
     def load_from_file(self, filename):
         """
@@ -122,21 +224,15 @@ class Config(object):
         """
         log.info('Loading Config object from file %s', filename)
 
-        try:
-            file_output = subprocess.check_output(['/usr/bin/vtysh', '-m', '-f', filename],
-                                                  stderr=subprocess.STDOUT)
-        except subprocess.CalledProcessError as e:
-            ve = VtyshMarkException(e)
-            ve.output = e.output
-            raise ve
+        file_output = self.vtysh.mark_file(filename)
 
-        for line in file_output.decode('utf-8').split('\n'):
+        for line in file_output.split('\n'):
             line = line.strip()
 
             # Compress duplicate whitespaces
             line = ' '.join(line.split())
 
-            if ":" in line:
+            if ":" in line and not "ipv6 add":
                 qv6_line = get_normalized_ipv6_line(line)
                 self.lines.append(qv6_line)
             else:
@@ -144,7 +240,7 @@ class Config(object):
 
         self.load_contexts()
 
-    def load_from_show_running(self):
+    def load_from_show_running(self, daemon):
         """
         Read running configuration and slurp it into internal memory
         The internal representation has been marked appropriately by passing it
@@ -152,16 +248,9 @@ class Config(object):
         """
         log.info('Loading Config object from vtysh show running')
 
-        try:
-            config_text = subprocess.check_output(
-                "/usr/bin/vtysh -c 'show run' | /usr/bin/tail -n +4 | /usr/bin/vtysh -m -f -",
-                shell=True, stderr=subprocess.STDOUT)
-        except subprocess.CalledProcessError as e:
-            ve = VtyshMarkException(e)
-            ve.output = e.output
-            raise ve
+        config_text = self.vtysh.mark_show_run(daemon)
 
-        for line in config_text.decode('utf-8').split('\n'):
+        for line in config_text.split('\n'):
             line = line.strip()
 
             if (line == 'Building configuration...' or
@@ -300,13 +389,11 @@ class Config(object):
 
         '''
           More fixups in user specification and what running config shows.
-          "null0" in routes must be replaced by Null0, and "blackhole" must
-          be replaced by Null0 as well.
+          "null0" in routes must be replaced by Null0.
         '''
         if (key[0].startswith('ip route') or key[0].startswith('ipv6 route') and
-                'null0' in key[0] or 'blackhole' in key[0]):
+                'null0' in key[0]):
             key[0] = re.sub(r'\s+null0(\s*$)', ' Null0', key[0])
-            key[0] = re.sub(r'\s+blackhole(\s*$)', ' Null0', key[0])
 
         if lines:
             if tuple(key) not in self.contexts:
@@ -395,8 +482,10 @@ end
         # is not the main router bgp block, but enabling multi-instance
         oneline_ctx_keywords = ("access-list ",
                                 "agentx",
+                                "allow-external-route-update",
                                 "bgp ",
                                 "debug ",
+                                "domainname ",
                                 "dump ",
                                 "enable ",
                                 "frr ",
@@ -404,7 +493,8 @@ end
                                 "ip ",
                                 "ipv6 ",
                                 "log ",
-                                "mpls",
+                                "mpls lsp",
+                                "mpls label",
                                 "no ",
                                 "password ",
                                 "ptm-enable",
@@ -412,7 +502,9 @@ end
                                 "service ",
                                 "table ",
                                 "username ",
-                                "zebra ")
+                                "zebra ",
+                                "vrrp autoconfigure",
+                                "evpn mh")
 
         for line in self.lines:
 
@@ -423,7 +515,12 @@ end
                 continue
 
             # one line contexts
-            if new_ctx is True and any(line.startswith(keyword) for keyword in oneline_ctx_keywords):
+            # there is one exception though: ldpd accepts a 'router-id' clause
+            # as part of its 'mpls ldp' config context. If we are processing
+            # ldp configuration and encounter a router-id we should NOT switch
+            # to a new context
+            if new_ctx is True and any(line.startswith(keyword) for keyword in oneline_ctx_keywords) and not (
+                ctx_keys and ctx_keys[0].startswith("mpls ldp") and line.startswith("router-id ")):
                 self.save_contexts(ctx_keys, current_context_lines)
 
                 # Start a new context
@@ -435,7 +532,7 @@ end
                 self.save_contexts(ctx_keys, current_context_lines)
                 new_ctx = True
 
-            elif line in ["end", "exit-vrf"]:
+            elif line == "end":
                 self.save_contexts(ctx_keys, current_context_lines)
                 log.debug('LINE %-50s: exiting old context, %-50s', line, ctx_keys)
 
@@ -445,6 +542,17 @@ end
                 ctx_keys = []
                 current_context_lines = []
 
+            elif line == "exit-vrf":
+                self.save_contexts(ctx_keys, current_context_lines)
+                current_context_lines.append(line)
+                log.debug('LINE %-50s: append to current_context_lines, %-50s', line, ctx_keys)
+
+                #Start a new context
+                new_ctx = True
+                main_ctx_key = []
+                ctx_keys = []
+                current_context_lines = []
+
             elif line in ["exit-address-family", "exit", "exit-vnc"]:
                 # if this exit is for address-family ipv4 unicast, ignore the pop
                 if main_ctx_key:
@@ -455,7 +563,7 @@ end
                     current_context_lines = []
                     log.debug('LINE %-50s: popping from subcontext to ctx%-50s', line, ctx_keys)
 
-            elif line == "exit-vni":
+            elif line in ["exit-vni", "exit-ldp-if"]:
                 if sub_main_ctx_key:
                     self.save_contexts(ctx_keys, current_context_lines)
 
@@ -477,7 +585,10 @@ end
             elif (line.startswith("address-family ") or
                   line.startswith("vnc defaults") or
                   line.startswith("vnc l2-group") or
-                  line.startswith("vnc nve-group")):
+                  line.startswith("vnc nve-group") or
+                  line.startswith("peer") or
+                  line.startswith("key ") or
+                  line.startswith("member pseudowire")):
                 main_ctx_key = []
 
                 # Save old context first
@@ -486,9 +597,9 @@ end
                 main_ctx_key = copy.deepcopy(ctx_keys)
                 log.debug('LINE %-50s: entering sub-context, append to ctx_keys', line)
 
-                if line == "address-family ipv6":
+                if line == "address-family ipv6" and not ctx_keys[0].startswith("mpls ldp"):
                     ctx_keys.append("address-family ipv6 unicast")
-                elif line == "address-family ipv4":
+                elif line == "address-family ipv4" and not ctx_keys[0].startswith("mpls ldp"):
                     ctx_keys.append("address-family ipv4 unicast")
                 elif line == "address-family evpn":
                     ctx_keys.append("address-family l2vpn evpn")
@@ -506,6 +617,18 @@ end
                 sub_main_ctx_key = copy.deepcopy(ctx_keys)
                 log.debug('LINE %-50s: entering sub-sub-context, append to ctx_keys', line)
                 ctx_keys.append(line)
+            
+            elif ((line.startswith("interface ") and
+                   len(ctx_keys) == 2 and
+                   ctx_keys[0].startswith('mpls ldp') and
+                   ctx_keys[1].startswith('address-family'))):
+
+                # Save old context first
+                self.save_contexts(ctx_keys, current_context_lines)
+                current_context_lines = []
+                sub_main_ctx_key = copy.deepcopy(ctx_keys)
+                log.debug('LINE %-50s: entering sub-sub-context, append to ctx_keys', line)
+                ctx_keys.append(line)
 
             else:
                 # Continuing in an existing context, add non-commented lines to it
@@ -516,58 +639,7 @@ end
         self.save_contexts(ctx_keys, current_context_lines)
 
 
-def line_to_vtysh_conft(ctx_keys, line, delete):
-    """
-    Return the vtysh command for the specified context line
-    """
-
-    cmd = []
-    cmd.append('vtysh')
-    cmd.append('-c')
-    cmd.append('conf t')
-
-    if line:
-        for ctx_key in ctx_keys:
-            cmd.append('-c')
-            cmd.append(ctx_key)
-
-        line = line.lstrip()
-
-        if delete:
-            cmd.append('-c')
-
-            if line.startswith('no '):
-                cmd.append('%s' % line[3:])
-            else:
-                cmd.append('no %s' % line)
-
-        else:
-            cmd.append('-c')
-            cmd.append(line)
-
-    # If line is None then we are typically deleting an entire
-    # context ('no router ospf' for example)
-    else:
-
-        if delete:
-
-            # Only put the 'no' on the last sub-context
-            for ctx_key in ctx_keys:
-                cmd.append('-c')
-
-                if ctx_key == ctx_keys[-1]:
-                    cmd.append('no %s' % ctx_key)
-                else:
-                    cmd.append('%s' % ctx_key)
-        else:
-            for ctx_key in ctx_keys:
-                cmd.append('-c')
-                cmd.append(ctx_key)
-
-    return cmd
-
-
-def line_for_vtysh_file(ctx_keys, line, delete):
+def lines_to_config(ctx_keys, line, delete):
     """
     Return the command as it would appear in frr.conf
     """
@@ -580,6 +652,10 @@ def line_for_vtysh_file(ctx_keys, line, delete):
         line = line.lstrip()
         indent = len(ctx_keys) * ' '
 
+        # There are some commands that are on by default so their "no" form will be
+        # displayed in the config.  "no bgp default ipv4-unicast" is one of these.
+        # If we need to remove this line we do so by adding "bgp default ipv4-unicast",
+        # not by doing a "no no bgp default ipv4-unicast"
         if delete:
             if line.startswith('no '):
                 cmd.append('%s%s' % (indent, line[3:]))
@@ -592,26 +668,17 @@ def line_for_vtysh_file(ctx_keys, line, delete):
     # If line is None then we are typically deleting an entire
     # context ('no router ospf' for example)
     else:
+        for i, ctx_key in enumerate(ctx_keys[:-1]):
+            cmd.append('%s%s' % (' ' * i, ctx_key))
+
+        # Only put the 'no' on the last sub-context
         if delete:
-
-            # Only put the 'no' on the last sub-context
-            for ctx_key in ctx_keys:
-
-                if ctx_key == ctx_keys[-1]:
-                    cmd.append('no %s' % ctx_key)
-                else:
-                    cmd.append('%s' % ctx_key)
+            if ctx_keys[-1].startswith('no '):
+                cmd.append('%s%s' % (' ' * (len(ctx_keys) - 1), ctx_keys[-1][3:]))
+            else:
+                cmd.append('%sno %s' % (' ' * (len(ctx_keys) - 1), ctx_keys[-1]))
         else:
-            for ctx_key in ctx_keys:
-                cmd.append(ctx_key)
-
-    cmd = '\n' + '\n'.join(cmd)
-
-    # There are some commands that are on by default so their "no" form will be
-    # displayed in the config.  "no bgp default ipv4-unicast" is one of these.
-    # If we need to remove this line we do so by adding "bgp default ipv4-unicast",
-    # not by doing a "no no bgp default ipv4-unicast"
-    cmd = cmd.replace('no no ', '')
+            cmd.append('%s%s' % (' ' * (len(ctx_keys) - 1), ctx_keys[-1]))
 
     return cmd
 
@@ -661,6 +728,36 @@ def line_exist(lines, target_ctx_keys, target_line, exact_match=True):
                     return True
     return False
 
+def check_for_exit_vrf(lines_to_add, lines_to_del):
+
+    # exit-vrf is a bit tricky.  If the new config is missing it but we
+    # have configs under a vrf, we need to add it at the end to do the
+    # right context changes.  If exit-vrf exists in both the running and
+    # new config, we cannot delete it or it will break context changes.
+    add_exit_vrf = False
+    index = 0
+
+    for (ctx_keys, line) in lines_to_add:
+        if add_exit_vrf == True:
+            if ctx_keys[0] != prior_ctx_key:
+                insert_key=(prior_ctx_key),
+                lines_to_add.insert(index, ((insert_key, "exit-vrf")))
+                add_exit_vrf = False
+
+        if ctx_keys[0].startswith('vrf') and line:
+            if line is not "exit-vrf":
+                add_exit_vrf = True
+                prior_ctx_key = (ctx_keys[0])
+            else:
+                add_exit_vrf = False
+        index+=1
+
+    for (ctx_keys, line) in lines_to_del:
+        if line == "exit-vrf":
+            if (line_exist(lines_to_add, ctx_keys, line)):
+                lines_to_del.remove((ctx_keys, line))
+
+    return (lines_to_add, lines_to_del)
 
 def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
 
@@ -732,6 +829,29 @@ def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
                         lines_to_add_to_del.append((ctx_keys, swpx_interface))
                         lines_to_add_to_del.append((tmp_ctx_keys, swpx_peergroup))
 
+                '''
+                Changing the bfd timers on neighbors is allowed without doing
+                a delete/add process. Since doing a "no neighbor blah bfd ..."
+                will cause the peer to bounce unnecessarily, just skip the delete
+                and just do the add.
+                '''
+                re_nbr_bfd_timers = re.search(r'neighbor (\S+) bfd (\S+) (\S+) (\S+)', line)
+
+                if re_nbr_bfd_timers:
+                    nbr = re_nbr_bfd_timers.group(1)
+                    bfd_nbr = "neighbor %s" % nbr
+                    bfd_search_string =  bfd_nbr + r' bfd (\S+) (\S+) (\S+)'
+
+                    for (ctx_keys, add_line) in lines_to_add:
+                        if ctx_keys[0].startswith('router bgp'):
+                            re_add_nbr_bfd_timers = re.search(bfd_search_string, add_line)
+
+                            if re_add_nbr_bfd_timers:
+                                found_add_bfd_nbr = line_exist(lines_to_add, ctx_keys, bfd_nbr, False)
+
+                                if found_add_bfd_nbr:
+                                    lines_to_del_to_del.append((ctx_keys, line))
+
                 '''
                 We changed how we display the neighbor interface command. Older
                 versions of frr would display the following:
@@ -879,6 +999,16 @@ def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
                     lines_to_del_to_del.append((ctx_keys, route_target_export_line))
                     lines_to_add_to_del.append((ctx_keys, route_target_both_line))
 
+        # Deleting static routes under a vrf can lead to time-outs if each is sent
+        # as separate vtysh -c commands. Change them from being in lines_to_del and
+        # put the "no" form in lines_to_add
+        if ctx_keys[0].startswith('vrf ') and line:
+            if (line.startswith('ip route') or
+                line.startswith('ipv6 route')):
+                add_cmd = ('no ' + line)
+                lines_to_add.append((ctx_keys, add_cmd))
+                lines_to_del_to_del.append((ctx_keys, line))
+
         if not deleted:
             found_add_line = line_exist(lines_to_add, ctx_keys, line)
 
@@ -933,6 +1063,7 @@ def ignore_unconfigurable_lines(lines_to_add, lines_to_del):
 
         if (ctx_keys[0].startswith('frr version') or
             ctx_keys[0].startswith('frr defaults') or
+            ctx_keys[0].startswith('username') or
             ctx_keys[0].startswith('password') or
             ctx_keys[0].startswith('line vty') or
 
@@ -941,7 +1072,7 @@ def ignore_unconfigurable_lines(lines_to_add, lines_to_del):
             # by removing this.
             ctx_keys[0].startswith('service integrated-vtysh-config')):
 
-            log.info("(%s, %s) cannot be removed" % (pformat(ctx_keys), line))
+            log.info('"%s" cannot be removed' % (ctx_keys[-1],))
             lines_to_del_to_del.append((ctx_keys, line))
 
     for (ctx_keys, line) in lines_to_del_to_del:
@@ -974,8 +1105,8 @@ def compare_context_objects(newconf, running):
                 delete_bgpd = True
                 lines_to_del.append((running_ctx_keys, None))
 
-            # We cannot do 'no interface' in FRR, and so deal with it
-            elif running_ctx_keys[0].startswith('interface'):
+            # We cannot do 'no interface' or 'no vrf' in FRR, and so deal with it
+            elif running_ctx_keys[0].startswith('interface') or running_ctx_keys[0].startswith('vrf'):
                 for line in running_ctx.lines:
                     lines_to_del.append((running_ctx_keys, line))
 
@@ -999,6 +1130,26 @@ def compare_context_objects(newconf, running):
                 for line in running_ctx.lines:
                     lines_to_del.append((running_ctx_keys, line))
 
+            # Some commands can happen at higher counts that make
+            # doing vtysh -c inefficient (and can time out.)  For
+            # these commands, instead of adding them to lines_to_del,
+            # add the "no " version to lines_to_add.
+            elif (running_ctx_keys[0].startswith('ip route') or
+                  running_ctx_keys[0].startswith('ipv6 route') or
+                  running_ctx_keys[0].startswith('access-list') or
+                  running_ctx_keys[0].startswith('ipv6 access-list') or
+                  running_ctx_keys[0].startswith('ip prefix-list') or
+                  running_ctx_keys[0].startswith('ipv6 prefix-list')):
+                add_cmd = ('no ' + running_ctx_keys[0],)
+                lines_to_add.append((add_cmd, None))
+
+            # if this an interface sub-subcontext in an address-family block in ldpd and
+            # we are already deleting the whole context, then ignore this
+            elif (len(running_ctx_keys) > 2 and running_ctx_keys[0].startswith('mpls ldp') and
+                  running_ctx_keys[1].startswith('address-family') and
+                  (running_ctx_keys[:2], None) in lines_to_del):
+                continue
+
             # Non-global context
             elif running_ctx_keys and not any("address-family" in key for key in running_ctx_keys):
                 lines_to_del.append((running_ctx_keys, None))
@@ -1034,38 +1185,13 @@ def compare_context_objects(newconf, running):
             for line in newconf_ctx.lines:
                 lines_to_add.append((newconf_ctx_keys, line))
 
+    (lines_to_add, lines_to_del) = check_for_exit_vrf(lines_to_add, lines_to_del)
     (lines_to_add, lines_to_del) = ignore_delete_re_add_lines(lines_to_add, lines_to_del)
     (lines_to_add, lines_to_del) = ignore_unconfigurable_lines(lines_to_add, lines_to_del)
 
     return (lines_to_add, lines_to_del)
 
 
-
-def vtysh_config_available():
-    """
-    Return False if no frr daemon is running or some other vtysh session is
-    in 'configuration terminal' mode which will prevent us from making any
-    configuration changes.
-    """
-
-    try:
-        cmd = ['/usr/bin/vtysh', '-c', 'conf t']
-        output = subprocess.check_output(cmd, stderr=subprocess.STDOUT).strip()
-
-        if 'VTY configuration is locked by other VTY' in output.decode('utf-8'):
-            print(output)
-            log.error("'%s' returned\n%s\n" % (' '.join(cmd), output))
-            return False
-
-    except subprocess.CalledProcessError as e:
-        msg = "vtysh could not connect with any frr daemons"
-        print(msg)
-        log.error(msg)
-        return False
-
-    return True
-
-
 if __name__ == '__main__':
     # Command line options
     parser = argparse.ArgumentParser(description='Dynamically apply diff in frr configs')
@@ -1073,18 +1199,28 @@ if __name__ == '__main__':
     group = parser.add_mutually_exclusive_group(required=True)
     group.add_argument('--reload', action='store_true', help='Apply the deltas', default=False)
     group.add_argument('--test', action='store_true', help='Show the deltas', default=False)
-    parser.add_argument('--debug', action='store_true', help='Enable debugs', default=False)
+    level_group = parser.add_mutually_exclusive_group()
+    level_group.add_argument('--debug', action='store_true',
+                             help='Enable debugs (synonym for --log-level=debug)', default=False)
+    level_group.add_argument('--log-level', help='Log level', default="info",
+                             choices=("critical", "error", "warning", "info", "debug"))
     parser.add_argument('--stdout', action='store_true', help='Log to STDOUT', default=False)
+    parser.add_argument('--pathspace', '-N', metavar='NAME', help='Reload specified path/namespace', default=None)
     parser.add_argument('filename', help='Location of new frr config file')
     parser.add_argument('--overwrite', action='store_true', help='Overwrite frr.conf with running config output', default=False)
+    parser.add_argument('--bindir', help='path to the vtysh executable', default='/usr/bin')
+    parser.add_argument('--confdir', help='path to the daemon config files', default='/etc/frr')
+    parser.add_argument('--rundir', help='path for the temp config file', default='/var/run/frr')
+    parser.add_argument('--vty_socket', help='socket to be used by vtysh to connect to the daemons', default=None)
+    parser.add_argument('--daemon', help='daemon for which want to replace the config', default='')
+
     args = parser.parse_args()
 
     # Logging
     # For --test log to stdout
     # For --reload log to /var/log/frr/frr-reload.log
     if args.test or args.stdout:
-        logging.basicConfig(level=logging.INFO,
-                            format='%(asctime)s %(levelname)5s: %(message)s')
+        logging.basicConfig(format='%(asctime)s %(levelname)5s: %(message)s')
 
         # Color the errors and warnings in red
         logging.addLevelName(logging.ERROR, "\033[91m  %s\033[0m" % logging.getLevelName(logging.ERROR))
@@ -1095,7 +1231,6 @@ if __name__ == '__main__':
             os.makedirs('/var/log/frr/')
 
         logging.basicConfig(filename='/var/log/frr/frr-reload.log',
-                            level=logging.INFO,
                             format='%(asctime)s %(levelname)5s: %(message)s')
 
     # argparse should prevent this from happening but just to be safe...
@@ -1103,21 +1238,58 @@ if __name__ == '__main__':
         raise Exception('Must specify --reload or --test')
     log = logging.getLogger(__name__)
 
+    if args.debug:
+        log.setLevel(logging.DEBUG)
+    else:
+        log.setLevel(args.log_level.upper())
+
+    if args.reload and not args.stdout:
+        # Additionally send errors and above to STDOUT, with no metadata,
+        # when we are logging to a file. This specifically does not follow
+        # args.log_level, and is analagous to behaviour in earlier versions
+        # which additionally logged most errors using print().
+
+        stdout_hdlr = logging.StreamHandler(sys.stdout)
+        stdout_hdlr.setLevel(logging.ERROR)
+        stdout_hdlr.setFormatter(logging.Formatter())
+        log.addHandler(stdout_hdlr)
+
     # Verify the new config file is valid
     if not os.path.isfile(args.filename):
-        msg = "Filename %s does not exist" % args.filename
-        print(msg)
-        log.error(msg)
+        log.error("Filename %s does not exist" % args.filename)
         sys.exit(1)
 
     if not os.path.getsize(args.filename):
-        msg = "Filename %s is an empty file" % args.filename
-        print(msg)
-        log.error(msg)
+        log.error("Filename %s is an empty file" % args.filename)
         sys.exit(1)
 
+    # Verify that confdir is correct
+    if not os.path.isdir(args.confdir):
+        log.error("Confdir %s is not a valid path" % args.confdir)
+        sys.exit(1)
+
+    # Verify that bindir is correct
+    if not os.path.isdir(args.bindir) or not os.path.isfile(args.bindir + '/vtysh'):
+        log.error("Bindir %s is not a valid path to vtysh" % args.bindir)
+        sys.exit(1)
+
+    # verify that the vty_socket, if specified, is valid
+    if args.vty_socket and not os.path.isdir(args.vty_socket):
+        log.error('vty_socket %s is not a valid path' % args.vty_socket)
+        sys.exit(1)
+
+    # verify that the daemon, if specified, is valid
+    if args.daemon and args.daemon not in ['zebra', 'bgpd', 'fabricd', 'isisd', 'ospf6d', 'ospfd', 'pbrd', 'pimd', 'ripd', 'ripngd', 'sharpd', 'staticd', 'vrrpd', 'ldpd']:
+        log.error("Daemon %s is not a valid option for 'show running-config'" % args.daemon)
+        sys.exit(1)
+
+    vtysh = Vtysh(args.bindir, args.confdir, args.vty_socket, args.pathspace)
+
     # Verify that 'service integrated-vtysh-config' is configured
-    vtysh_filename = '/etc/frr/vtysh.conf'
+    if args.pathspace:
+        vtysh_filename = args.confdir + '/' + args.pathspace + '/vtysh.conf'
+    else:
+        vtysh_filename = args.confdir + '/vtysh.conf'
     service_integrated_vtysh_config = True
 
     if os.path.isfile(vtysh_filename):
@@ -1129,31 +1301,30 @@ if __name__ == '__main__':
                     service_integrated_vtysh_config = False
                     break
 
-    if not service_integrated_vtysh_config:
-        msg = "'service integrated-vtysh-config' is not configured, this is required for 'service frr reload'"
-        print(msg)
-        log.error(msg)
+    if not service_integrated_vtysh_config and not args.daemon:
+        log.error("'service integrated-vtysh-config' is not configured, this is required for 'service frr reload'")
         sys.exit(1)
 
-    if args.debug:
-        log.setLevel(logging.DEBUG)
-
     log.info('Called via "%s"', str(args))
 
     # Create a Config object from the config generated by newconf
-    newconf = Config()
-    newconf.load_from_file(args.filename)
-    reload_ok = True
+    newconf = Config(vtysh)
+    try:
+        newconf.load_from_file(args.filename)
+        reload_ok = True
+    except VtyshException as ve:
+        log.error("vtysh failed to process new configuration: {}".format(ve))
+        reload_ok = False
 
     if args.test:
 
         # Create a Config object from the running config
-        running = Config()
+        running = Config(vtysh)
 
         if args.input:
             running.load_from_file(args.input)
         else:
-            running.load_from_show_running()
+            running.load_from_show_running(args.daemon)
 
         (lines_to_add, lines_to_del) = compare_context_objects(newconf, running)
         lines_to_configure = []
@@ -1167,7 +1338,7 @@ if __name__ == '__main__':
                 if line == '!':
                     continue
 
-                cmd = line_for_vtysh_file(ctx_keys, line, True)
+                cmd = '\n'.join(lines_to_config(ctx_keys, line, True))
                 lines_to_configure.append(cmd)
                 print(cmd)
 
@@ -1180,14 +1351,14 @@ if __name__ == '__main__':
                 if line == '!':
                     continue
 
-                cmd = line_for_vtysh_file(ctx_keys, line, False)
+                cmd = '\n'.join(lines_to_config(ctx_keys, line, False))
                 lines_to_configure.append(cmd)
                 print(cmd)
 
     elif args.reload:
 
         # We will not be able to do anything, go ahead and exit(1)
-        if not vtysh_config_available():
+        if not vtysh.is_config_available():
             sys.exit(1)
 
         log.debug('New Frr Config\n%s', newconf.get_lines())
@@ -1230,8 +1401,8 @@ if __name__ == '__main__':
         lines_to_add_first_pass = []
 
         for x in range(2):
-            running = Config()
-            running.load_from_show_running()
+            running = Config(vtysh)
+            running.load_from_show_running(args.daemon)
             log.debug('Running Frr Config (Pass #%d)\n%s', x, running.get_lines())
 
             (lines_to_add, lines_to_del) = compare_context_objects(newconf, running)
@@ -1263,7 +1434,7 @@ if __name__ == '__main__':
                     # 'no' commands are tricky, we can't just put them in a file and
                     # vtysh -f that file. See the next comment for an explanation
                     # of their quirks
-                    cmd = line_to_vtysh_conft(ctx_keys, line, True)
+                    cmd = lines_to_config(ctx_keys, line, True)
                     original_cmd = cmd
 
                     # Some commands in frr are picky about taking a "no" of the entire line.
@@ -1282,9 +1453,9 @@ if __name__ == '__main__':
 
                     while True:
                         try:
-                            _ = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
+                            vtysh(['configure'] + cmd)
 
-                        except subprocess.CalledProcessError:
+                        except VtyshException:
 
                             # - Pull the last entry from cmd (this would be
                             #   'no ip ospf authentication message-digest 1.1.1.1' in
@@ -1294,7 +1465,7 @@ if __name__ == '__main__':
                             last_arg = cmd[-1].split(' ')
 
                             if len(last_arg) <= 2:
-                                log.error('"%s" we failed to remove this command', original_cmd)
+                                log.error('"%s" we failed to remove this command', ' -- '.join(original_cmd))
                                 break
 
                             new_last_arg = last_arg[0:-1]
@@ -1311,7 +1482,12 @@ if __name__ == '__main__':
                     if line == '!':
                         continue
 
-                    cmd = line_for_vtysh_file(ctx_keys, line, False)
+                    # Don't run "no" commands twice since they can error
+                    # out the second time due to first deletion
+                    if x == 1 and ctx_keys[0].startswith('no '):
+                        continue
+
+                    cmd = '\n'.join(lines_to_config(ctx_keys, line, False)) + '\n'
                     lines_to_configure.append(cmd)
 
                 if lines_to_configure:
@@ -1319,7 +1495,7 @@ if __name__ == '__main__':
                                             string.ascii_uppercase +
                                             string.digits) for _ in range(6))
 
-                    filename = "/var/run/frr/reload-%s.txt" % random_string
+                    filename = args.rundir + "/reload-%s.txt" % random_string
                     log.info("%s content\n%s" % (filename, pformat(lines_to_configure)))
 
                     with open(filename, 'w') as fh:
@@ -1327,15 +1503,16 @@ if __name__ == '__main__':
                             fh.write(line + '\n')
 
                     try:
-                        subprocess.check_output(['/usr/bin/vtysh', '-f', filename], stderr=subprocess.STDOUT)
-                    except subprocess.CalledProcessError as e:
-                        log.warning("frr-reload.py failed due to\n%s" % e.output)
+                        vtysh.exec_file(filename)
+                    except VtyshException as e:
+                        log.warning("frr-reload.py failed due to\n%s" % e.args)
                         reload_ok = False
                     os.unlink(filename)
 
         # Make these changes persistent
-        if args.overwrite or args.filename != '/etc/frr/frr.conf':
-            subprocess.call(['/usr/bin/vtysh', '-c', 'write'])
+        target = str(args.confdir + '/frr.conf')
+        if args.overwrite or (not args.daemon and args.filename != target):
+            vtysh('write')
 
     if not reload_ok:
         sys.exit(1)
